# -*- Mode: Python; py-indent-offset: 4 -*-
# Authors: Nik Kim <fafhrd@datacom.kz> 

"""Formulator Form object that come from the filesystem."""
__version__='$Revision: 0.2 $'[11:-2]

import string, os
import Globals
from DateTime import DateTime
from AccessControl import ClassSecurityInfo

from loutils import _dtmldir
from Products.CMFCore.CMFCorePermissions import ViewManagementScreens, View
from Products.CMFCore.FSObject import FSObject
from Products.CMFCore.DirectoryView \
     import registerFileExtension, registerMetaType, expandpath

from Products.Formulator.Form import ZMIForm
from Products.Formulator.Field import Field
from Products.Formulator.TALESField import TALESMethod

from xmlreader import readData

class FSForm(FSObject, ZMIForm):
    """FSForm."""

    meta_type = 'Filesystem Formulator Form'

    def manage_options(self):
        """ """
        o = [{'label':'Customize', 'action':'manage_castomize'}]
        o += ZMIForm.manage_options[1:]
        return o
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(View)

    i18n_domain="portal_forms"
    show_buttons=1
    view_template = 'fs_form_view'

    def __init__(self, id, filepath, fullname=None, properties=None):
        FSObject.__init__(self, id, filepath, fullname, properties)

    security.declareProtected(ViewManagementScreens, 'manage_castomize')
    manage_castomize = Globals.DTMLFile('cust_form', _dtmldir)

    def _createZODBClone(self):
        return None

    def _readFile(self, reparse):
        data = readData(expandpath(self._filepath))
        form = data['form']

        ZMIForm.__init__(self, self.id, form['title'])

        # set form settings
        settings = {}
        settings.update(form)
        for r_id in ['title', 'description', 'groups']:
            try:
                del settings[r_id]
            except: pass
            
        for key, value in settings.items():
            setattr(self, key, value)

        has_default = 0
        # create groups
        for group in form['groups']:
            group_title = group.get('title', '')
            if group_title == 'Default':
                has_default = 1
            
            self.add_group(group_title)

            for item in group.get('fields', []):
                # create field
                self.manage_addField(item['id'], '', item['type'])
                field = self._getOb(item['id'])
                if group_title != 'Default':
                    self.move_field_group([item['id']], 'Default', group_title)

                # set values
                try:
                    for key, value in item.get('values', {}).items():
                        field.values[key] = value
                except: pass
                
                # set tales
                try:
                    for key, value in item.get('tales', {}).items():
                        field.tales[key] = TALESMethod(value)
                except: pass

        # delete Default group
        if not has_default:
            self.move_group_down('Default')
            self.remove_group('Default')

            
    def index_html(self, REQUEST=None):
        """ """
        return getattr(self.aq_parent, self.view_template)(form=self)

    def Title(self):
        """ """
        return self.title


Globals.InitializeClass(FSForm)

registerFileExtension('form', FSForm)
registerMetaType('FSForm', FSForm)

# i need 'here' in TALES
def new_get_value(self, id):
    """Get value for id."""
    # FIXME: backwards compat hack to make sure tales dict exists
    if not hasattr(self, 'tales'):
        self.tales = {}
        
    tales_expr = self.tales.get(id, "")
    if tales_expr:
        value = tales_expr.__of__(self)(field=self, form=self.aq_parent,
                                        here=self.aq_parent.aq_parent)
    else:
        # FIXME: backwards compat hack to make sure overrides dict exists
        if not hasattr(self, 'overrides'):
            self.overrides = {}
                
        override = self.overrides.get(id, "")
        if override:
            # call wrapped method to get answer
            value = override.__of__(self)()
        else:
            # get normal value
            value = self.get_orig_value(id)

    # if normal value is a callable itself, wrap it
    if callable(value):
        return value.__of__(self)
    else:
        return value


Field.get_value = new_get_value
