##############################################################################
#
# Copyright (c) 2002 Nik Kim <fafhrd@mindless.com> All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

import traceback
from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import SimpleItemWithProperties
from Products.CMFCore.CMFCorePermissions import View, ModifyPortalContent

try:
    _lo_attrs = 1
    from Products.CMFCore.LocaleAttributes import LocaleAttributes
except:
    _lo_attrs = 0
    class LocaleAttributes: pass


class Group(SimpleItemWithProperties, LocaleAttributes):
    """ """
    meta_type = 'Form Group'

    security = ClassSecurityInfo()

    if _lo_attrs:
        __locale_attrs__ = {'title':''}
    else:
        title = ''

    fields = []

    def __init__(self, title=''):
        self.title = title

    security.declareProtected(ModifyPortalContent, 'edit')
    def edit(self, title=''):
        """ """
        self.title = title

    security.declarePublic('Title')
    def Title(self):
        """ """
        return self.title

    security.declareProtected(ModifyPortalContent, 'addFields')
    def addFields(self, fields=[]):
        """ """
        self.fields = self.fields + fields

    security.declareProtected(ModifyPortalContent, 'delField')
    def delField(self, field):
        """ """
        try:
            idx = 0
            fs = self.fields
            f_id = field.getId()
            while idx < len(fs):
                if f_id == fs[idx].getId():
                    del fs[idx]
                    continue
                idx += 1
            self.fields = fs
        except:
            traceback.print_exc()

    security.declareProtected(ModifyPortalContent, 'delFields')
    def delFields(self, fields=[]):
        """ """
        fs = self.fields
        for idx in fields:
            try:
                del fs[idx]
            except:
                pass
        self.fields = fs
    
    security.declareProtected(View, 'Fields')
    def Fields(self):
        """ """
        form = self.aq_parent
        return [f.__of__(form) for f in self.fields]

    security.declarePublic('Form')
    def Form(self):
        """ """
        return self.aq_parent

    security.declareProtected(ModifyPortalContent, 'moveFieldsUp')
    def moveFieldsUp(self, fields):
        f_list = self.fields
        for i in fields:
            if i == 0:
                continue
            f_list[i], f_list[i - 1] = f_list[i - 1], f_list[i]
        self.fields = f_list
        return 1

    security.declareProtected(ModifyPortalContent, 'moveFieldsDown')
    def moveFieldsDown(self, fields):
        f_list = self.fields
        f_len = len(f_list)
        for i in fields:
            if i == f_len - 1:
                return 0
            f_list[i], f_list[i + 1] = f_list[i + 1], f_list[i]
        self.fields = f_list
        return 1

InitializeClass(Group)
